@@ -163,24 +163,6 @@ char *auditRole = NULL;
 #define AUDIT_TYPE_OBJECT   "OBJECT"
 #define AUDIT_TYPE_SESSION  "SESSION"
 
-/*
- * Command, used for SELECT/DML and function calls.
- *
- * We hook into the executor, but we do not have access to the parsetree there.
- * Therefore we can't simply call CreateCommandTag() to get the command and have
- * to build it ourselves based on what information we do have.
- *
- * These should be updated if new commands are added to what the exectuor
- * currently handles.  Note that most of the interesting commands do not go
- * through the executor but rather ProcessUtility, where we have the parsetree.
- */
-#define COMMAND_SELECT      "SELECT"
-#define COMMAND_INSERT      "INSERT"
-#define COMMAND_UPDATE      "UPDATE"
-#define COMMAND_DELETE      "DELETE"
-#define COMMAND_EXECUTE     "EXECUTE"
-#define COMMAND_UNKNOWN     "UNKNOWN"
-
 /*
  * Object type, used for SELECT/DML statements and function calls.
  *
@@ -205,17 +187,6 @@ char *auditRole = NULL;
 
 #define OBJECT_TYPE_UNKNOWN         "UNKNOWN"
 
-/*
- * String constants for testing role commands.  Rename and drop role statements
- * are assigned the nodeTag T_RenameStmt and T_DropStmt respectively.  This is
- * not very useful for classification, so we resort to comparing strings
- * against the result of CreateCommandTag(parsetree).
- */
-#define COMMAND_ALTER_ROLE          "ALTER ROLE"
-#define COMMAND_DROP_ROLE           "DROP ROLE"
-#define COMMAND_GRANT               "GRANT"
-#define COMMAND_REVOKE              "REVOKE"
-
 /*
  * String constants used for redacting text after the password token in
  * CREATE/ALTER ROLE commands.
@@ -236,7 +207,7 @@ typedef struct
     LogStmtLevel logStmtLevel;  /* From GetCommandLogLevel when possible,
                                    generated when not. */
     NodeTag commandTag;         /* same here */
-    const char *command;        /* same here */
+    int command;                /* same here */
     const char *objectType;     /* From event trigger when possible,
                                    generated when not. */
     char *objectName;           /* Fully qualified object identification */
@@ -563,6 +534,8 @@ log_audit_event(AuditEventStackItem *stackItem)
                         }
                     }
 
+                /* Fall through */
+
                 /* Classify role statements */
                 case T_GrantStmt:
                 case T_GrantRoleStmt:
@@ -580,10 +553,8 @@ log_audit_event(AuditEventStackItem *stackItem)
                  */
                 case T_RenameStmt:
                 case T_DropStmt:
-                    if (pg_strcasecmp(stackItem->auditEvent.command,
-                                      COMMAND_ALTER_ROLE) == 0 ||
-                        pg_strcasecmp(stackItem->auditEvent.command,
-                                      COMMAND_DROP_ROLE) == 0)
+                    if (stackItem->auditEvent.command == CMDTAG_ALTER_ROLE ||
+                        stackItem->auditEvent.command == CMDTAG_DROP_ROLE)
                     {
                         className = CLASS_ROLE;
                         class = LOG_ROLE;
@@ -671,7 +642,7 @@ log_audit_event(AuditEventStackItem *stackItem)
      * this string is everything else.
      */
     initStringInfo(&auditStr);
-    append_valid_csv(&auditStr, stackItem->auditEvent.command);
+    append_valid_csv(&auditStr, GetCommandTagName(stackItem->auditEvent.command));
 
     appendStringInfoCharMacro(&auditStr, ',');
     append_valid_csv(&auditStr, stackItem->auditEvent.objectType);
@@ -1030,37 +1001,40 @@ log_select_dml(Oid auditOid, List *rangeTabls)
         /*
          * We don't have access to the parsetree here, so we have to generate
          * the node type, object type, and command tag by decoding
-         * rte->requiredPerms and rte->relkind.
+         * rte->requiredPerms and rte->relkind. For updates we also check 
+         * rellockmode so that only true UPDATE commands (not
+         * SELECT FOR UPDATE, etc.) are logged as UPDATE.
          */
         if (rte->requiredPerms & ACL_INSERT)
         {
             auditEventStack->auditEvent.logStmtLevel = LOGSTMT_MOD;
             auditEventStack->auditEvent.commandTag = T_InsertStmt;
-            auditEventStack->auditEvent.command = COMMAND_INSERT;
+            auditEventStack->auditEvent.command = CMDTAG_INSERT;
         }
-        else if (rte->requiredPerms & ACL_UPDATE)
+        else if (rte->requiredPerms & ACL_UPDATE &&
+                 rte->rellockmode >= RowExclusiveLock)
         {
             auditEventStack->auditEvent.logStmtLevel = LOGSTMT_MOD;
             auditEventStack->auditEvent.commandTag = T_UpdateStmt;
-            auditEventStack->auditEvent.command = COMMAND_UPDATE;
+            auditEventStack->auditEvent.command = CMDTAG_UPDATE;
         }
         else if (rte->requiredPerms & ACL_DELETE)
         {
             auditEventStack->auditEvent.logStmtLevel = LOGSTMT_MOD;
             auditEventStack->auditEvent.commandTag = T_DeleteStmt;
-            auditEventStack->auditEvent.command = COMMAND_DELETE;
+            auditEventStack->auditEvent.command = CMDTAG_DELETE;
         }
         else if (rte->requiredPerms & ACL_SELECT)
         {
             auditEventStack->auditEvent.logStmtLevel = LOGSTMT_ALL;
             auditEventStack->auditEvent.commandTag = T_SelectStmt;
-            auditEventStack->auditEvent.command = COMMAND_SELECT;
+            auditEventStack->auditEvent.command = CMDTAG_SELECT;
         }
         else
         {
             auditEventStack->auditEvent.logStmtLevel = LOGSTMT_ALL;
             auditEventStack->auditEvent.commandTag = T_Invalid;
-            auditEventStack->auditEvent.command = COMMAND_UNKNOWN;
+            auditEventStack->auditEvent.command = CMDTAG_UNKNOWN;
         }
 
         /* Use the relation type to assign object type */
@@ -1234,7 +1208,7 @@ log_function_execute(Oid objectId)
     /* Log the function call */
     stackItem->auditEvent.logStmtLevel = LOGSTMT_ALL;
     stackItem->auditEvent.commandTag = T_DoStmt;
-    stackItem->auditEvent.command = COMMAND_EXECUTE;
+    stackItem->auditEvent.command = CMDTAG_EXECUTE;
     stackItem->auditEvent.objectType = OBJECT_TYPE_FUNCTION;
     stackItem->auditEvent.commandText = stackItem->next->auditEvent.commandText;
 
@@ -1273,31 +1247,31 @@ pgaudit_ExecutorStart_hook(QueryDesc *queryDesc, int eflags)
             case CMD_SELECT:
                 stackItem->auditEvent.logStmtLevel = LOGSTMT_ALL;
                 stackItem->auditEvent.commandTag = T_SelectStmt;
-                stackItem->auditEvent.command = COMMAND_SELECT;
+                stackItem->auditEvent.command = CMDTAG_SELECT;
                 break;
 
             case CMD_INSERT:
                 stackItem->auditEvent.logStmtLevel = LOGSTMT_MOD;
                 stackItem->auditEvent.commandTag = T_InsertStmt;
-                stackItem->auditEvent.command = COMMAND_INSERT;
+                stackItem->auditEvent.command = CMDTAG_INSERT;
                 break;
 
             case CMD_UPDATE:
                 stackItem->auditEvent.logStmtLevel = LOGSTMT_MOD;
                 stackItem->auditEvent.commandTag = T_UpdateStmt;
-                stackItem->auditEvent.command = COMMAND_UPDATE;
+                stackItem->auditEvent.command = CMDTAG_UPDATE;
                 break;
 
             case CMD_DELETE:
                 stackItem->auditEvent.logStmtLevel = LOGSTMT_MOD;
                 stackItem->auditEvent.commandTag = T_DeleteStmt;
-                stackItem->auditEvent.command = COMMAND_DELETE;
+                stackItem->auditEvent.command = CMDTAG_DELETE;
                 break;
 
             default:
                 stackItem->auditEvent.logStmtLevel = LOGSTMT_ALL;
                 stackItem->auditEvent.commandTag = T_Invalid;
-                stackItem->auditEvent.command = COMMAND_UNKNOWN;
+                stackItem->auditEvent.command = CMDTAG_UNKNOWN;
                 break;
         }
 
@@ -1358,7 +1332,7 @@ pgaudit_ProcessUtility_hook(PlannedStmt *pstmt,
                             ParamListInfo params,
                             QueryEnvironment *queryEnv,
                             DestReceiver *dest,
-                            char *completionTag)
+                            QueryCompletion *qc)
 {
     AuditEventStackItem *stackItem = NULL;
     int64 stackId = 0;
@@ -1432,10 +1406,10 @@ pgaudit_ProcessUtility_hook(PlannedStmt *pstmt,
     /* Call the standard process utility chain. */
     if (next_ProcessUtility_hook)
         (*next_ProcessUtility_hook) (pstmt, queryString, context, params,
-                                     queryEnv, dest, completionTag);
+                                     queryEnv, dest, qc);
     else
         standard_ProcessUtility(pstmt, queryString, context, params,
-                                queryEnv, dest, completionTag);
+                                queryEnv, dest, qc);
 
     /*
      * Process the audit event if there is one.  Also check that this event
@@ -1565,7 +1539,7 @@ pgaudit_ddl_command_end(PG_FUNCTION_ARGS)
         auditEventStack->auditEvent.objectName =
             SPI_getvalue(spiTuple, spiTupDesc, 2);
         auditEventStack->auditEvent.command =
-            SPI_getvalue(spiTuple, spiTupDesc, 3);
+            GetCommandTagEnum(SPI_getvalue(spiTuple, spiTupDesc, 3));
 
         auditEventStack->auditEvent.logged = false;
 
@@ -1573,8 +1547,8 @@ pgaudit_ddl_command_end(PG_FUNCTION_ARGS)
          * Identify grant/revoke commands - these are the only non-DDL class
          * commands that should be coming through the event triggers.
          */
-        if (pg_strcasecmp(auditEventStack->auditEvent.command, COMMAND_GRANT) == 0 ||
-            pg_strcasecmp(auditEventStack->auditEvent.command, COMMAND_REVOKE) == 0)
+        if (auditEventStack->auditEvent.command == CMDTAG_GRANT ||
+            auditEventStack->auditEvent.command == CMDTAG_REVOKE)
         {
             NodeTag currentCommandTag = auditEventStack->auditEvent.commandTag;
 
