@@ -414,6 +414,15 @@ SELECT password
 UPDATE account
    SET description = 'yada, yada';
 
+--
+-- Object logged because of:
+--     select (password) on account (in the where clause)
+-- Session logged on all tables because log = read and log_relation = on
+SELECT *
+  FROM account
+ WHERE password = 'HASH2'
+   FOR UPDATE;
+
 --
 -- Object logged because of:
 -- select (password) on account (in the where clause)
@@ -765,7 +774,7 @@ CREATE TRIGGER bbb_insert_trg
 	AFTER INSERT ON bbb
 	FOR EACH ROW EXECUTE PROCEDURE bbb_insert();
 
-GRANT SELECT
+GRANT SELECT, UPDATE
    ON aaa
    TO auditor;
 
